"""Willian Notifications -- In-app notification system with PostgreSQL storage."""

from notifications.config import NotificationConfig, init_notifications
from notifications.models import (
    CreateNotification,
    Notification,
    NotificationPreferences,
)
from notifications.router import NotificationRouter
from notifications.service import (
    get_notifications,
    mark_all_read,
    mark_read,
    send_notification,
)

__all__ = [
    "CreateNotification",
    "Notification",
    "NotificationConfig",
    "NotificationPreferences",
    "NotificationRouter",
    "get_notifications",
    "init_notifications",
    "mark_all_read",
    "mark_read",
    "send_notification",
]
